@@ -73,10 +73,12 @@ namespace xpyt
         nl::json internal_request_impl(const nl::json& content) override;
 
         void redirect_output();
-        void redirect_display(bool install_hook=true);
-        void load_extensions();
 
+        py::object m_ipython_shell_app;
+        py::object m_ipython_shell;
         py::object m_displayhook;
+        py::object m_logger;
+        py::object m_terminal_stream;
 
         // The interpreter has the same scope as a `gil_scoped_release` instance
         // so that the GIL is not held by default, it will only be held when the
@@ -92,7 +94,7 @@ namespace xpyt
         bool m_release_gil_at_startup = true;
         gil_scoped_release_ptr m_release_gil = nullptr;
 
-        bool m_has_ipython;
+        bool m_redirect_display_enabled;
     };
 }
 
