@@ -29,10 +29,15 @@ namespace xpyt
         std::vector<std::string> m_traceback;
     };
 
+    XEUS_PYTHON_API py::module get_traceback_module();
+
     XEUS_PYTHON_API void register_filename_mapping(const std::string& filename, int execution_count);
-    
+
     XEUS_PYTHON_API XPYT_FORCE_PYBIND11_EXPORT
     xerror extract_error(py::error_already_set& error);
+
+    XEUS_PYTHON_API XPYT_FORCE_PYBIND11_EXPORT
+    xerror extract_error(const py::object& type, const py::object& value, const py::object& traceback);
 }
 
 #endif
