@@ -8,6 +8,7 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
+#include <cctype>
 #include <cstdlib>
 #include <fstream>
 #include <string>
@@ -28,6 +29,7 @@
 #include "xeus/xsystem.hpp"
 
 #include "xeus-python/xdebugger.hpp"
+#include "xeus-python/xutils.hpp"
 #include "xdebugpy_client.hpp"
 #include "xinternal_utils.hpp"
 
@@ -59,6 +61,7 @@ namespace xpyt
     {
         m_debugpy_port = xeus::find_free_port(100, 5678, 5900);
         register_request_handler("inspectVariables", std::bind(&debugger::inspect_variables_request, this, _1), false);
+        register_request_handler("richInspectVariables", std::bind(&debugger::rich_inspect_variables_request, this, _1), false);
         register_request_handler("attach", std::bind(&debugger::attach_request, this, _1), true);
         register_request_handler("configurationDone", std::bind(&debugger::configuration_done_request, this, _1), true);
     }
@@ -69,6 +72,48 @@ namespace xpyt
         p_debugpy_client = nullptr;
     }
 
+    namespace
+    {
+        using name_list = std::vector<std::string>;
+
+        const name_list& get_excluded_variables()
+        {
+            static name_list l =
+            { 
+                "__name__",
+                "__doc__",
+                "__package__",
+                "__loader__",
+                "__spec__",
+                "__annotations__",
+                "__builtins__",
+                "__builtin__",
+                "display",
+                "get_ipython",
+                "debugpy",
+                "exit",
+                "quit",
+                "In",
+                "Out",
+                "_oh",
+                "_dh",
+                "_",
+                "__",
+                "___"
+            };
+            return l;
+        };
+
+        bool keep_variable(const std::string& var_name)
+        {
+            const name_list& l = get_excluded_variables();
+            bool res = var_name.substr(0u, 2u) != "_i";
+            res = res && !(var_name[0] == '_' && std::isdigit(var_name[1]));
+            res = res && std::find(l.cbegin(), l.cend(), var_name) == l.cend();
+            return res;
+        }
+    }
+
     nl::json debugger::inspect_variables_request(const nl::json& message)
     {
         py::gil_scoped_acquire acquire;
@@ -78,17 +123,25 @@ namespace xpyt
         for (const py::handle& key : variables)
         {
             nl::json json_var = nl::json::object();
-            json_var["name"] = py::str(key);
-            json_var["variablesReference"] = 0;
-            try
+            std::string var_name = py::str(key);
+            if (keep_variable(var_name))
             {
-                json_var["value"] = variables[key];
+                json_var["name"] = var_name;
+                json_var["variablesReference"] = 0;
+                try
+                {
+                    json_var["value"] = variables[key];
+                }
+                catch(std::exception&)
+                {
+                    json_var["value"] = py::repr(variables[key]);
+                }
+                std::string var_type = py::str(variables[key].get_type());
+                size_t size = var_type.size();
+                std::string var_trunc_type = var_type.substr(size_t(8), size - 10u);
+                json_var["type"] = var_trunc_type;
+                json_vars.push_back(json_var);
             }
-            catch(std::exception&)
-            {
-                json_var["value"] = py::repr(variables[key]);
-            }
-            json_vars.push_back(json_var);
         }
 
         nl::json reply = {
@@ -104,6 +157,73 @@ namespace xpyt
         return reply;
     }
 
+    nl::json debugger::rich_inspect_variables_request(const nl::json& message)
+    {
+
+
+        std::string var_name = message["arguments"]["variableName"].get<std::string>();
+        std::string var_repr_data = var_name + "_repr_data";
+        std::string var_repr_metadata = var_name + "_repr_metada";
+
+        if (base_type::get_stopped_threads().empty())
+        {
+            // The code did not hit a breakpoint, we use the interpreter 
+            // to get the rich reprensentation of the variable
+            std::string code = "from IPython import get_ipython;";
+            code += var_repr_data + ',' + var_repr_metadata + "= get_ipython().display_formatter.format(" + var_name + ")";
+            py::gil_scoped_acquire acquire;
+            exec(py::str(code));
+        }
+        else
+        {   
+            // The code has stopped on a breakpoint, we use the setExpression request
+            // to get the rich representation of the variable
+            std::string lvalue = var_repr_data + ',' + var_repr_metadata;
+            std::string code = "get_ipython().display_formatter.format(" + var_name + ")";
+            int frame_id = message["arguments"]["frameId"].get<int>();
+            int seq = message["seq"].get<int>();
+            nl::json request = {
+                {"type", "request"},
+                {"command", "setExpression"},
+                {"seq", seq+1},
+                {"arguments", {
+                    {"expression", lvalue},
+                    {"value", code},
+                    {"frameId", frame_id}
+                }}
+            };
+            forward_message(request);
+        }
+
+        nl::json reply = {
+            {"type", "response"},
+            {"request_seq", message["seq"]},
+            {"success", false},
+            {"command", message["command"]}
+        };
+
+        py::gil_scoped_acquire acquire;
+        py::object variables = py::globals();
+        py::object repr_data = variables[py::str(var_repr_data)];
+        py::object repr_metadata = variables[py::str(var_repr_metadata)];
+        nl::json body = {
+            {"data", {}},
+            {"metadata", {}}
+        };
+        for (const py::handle& key : repr_data)
+        {
+            std::string data_key = py::str(key);
+            body["data"][data_key] = repr_data[key];
+            if (repr_metadata.contains(key))
+            {
+                body["metadata"][data_key] = repr_metadata[key];
+            }
+        }
+        reply["body"] = body;
+        reply["success"] = true;
+        return reply;
+    }
+
     nl::json debugger::attach_request(const nl::json& message)
     {
         nl::json new_message = message;
@@ -216,7 +336,9 @@ namespace xpyt
     {
         return xeus::xdebugger_info(xeus::get_tmp_hash_seed(),
                                     get_tmp_prefix(),
-                                    get_tmp_suffix());
+                                    get_tmp_suffix(),
+                                    true,
+                                    {"Python Exceptions"});
     }
 
     std::string debugger::get_cell_temporary_file(const std::string& code) const
