@@ -314,6 +314,47 @@ nl::json make_inspect_variables_request(int seq)
     return req;
 }
 
+nl::json make_rich_inspect_variables_request(int seq, const std::string& var_name)
+{
+    nl::json req = {
+        {"type", "request"},
+        {"seq", seq},
+        {"command", "richInspectVariables"},
+        {"arguments", {
+            {"variableName", var_name},
+        }}
+    };
+    return req;
+}
+
+nl::json make_rich_inspect_variables_request(int seq, const std::string& var_name, int frame_id)
+{
+    nl::json req = make_rich_inspect_variables_request(seq, var_name);
+    req["arguments"]["frameId"] = frame_id;
+    return req;
+}
+
+nl::json make_exception_breakpoint_request(int seq)
+{
+    nl::json except_option = {
+        {"path", nl::json::array({
+                nl::json::object({{"names", nl::json::array({"Python Exceptions"})}})
+        })},
+        {"breakMode", "always"}
+    };
+    nl::json options = nl::json::array({except_option, except_option});
+    nl::json req = {
+        {"type", "request"},
+        {"seq", seq},
+        {"command", "setExceptionBreakpoints"},
+        {"arguments", {
+            {"filters", {"raised", "uncaught"}},
+            {"exceptionOptions", options}
+        }}
+    };
+    return req;
+}
+
 /*******************
  * debugger_client *
  *******************/
@@ -332,12 +373,14 @@ public:
     bool test_external_set_breakpoints();
     bool test_external_next_continue();
     bool test_set_breakpoints();
+    bool test_set_exception_breakpoints();
     bool test_source();
     bool test_next_continue();
     bool test_step_in();
     bool test_stack_trace();
     bool test_debug_info();
     bool test_inspect_variables();
+    bool test_rich_inspect_variables();
     bool test_variables();
     void shutdown();
 
@@ -346,6 +389,7 @@ private:
     nl::json attach();
     nl::json set_external_breakpoints();
     nl::json set_breakpoints();
+    nl::json set_exception_breakpoints();
 
     std::string get_external_path();
     void dump_external_file();
@@ -447,6 +491,22 @@ bool debugger_client::test_set_breakpoints()
     return rep["content"]["body"].size() != 0;
 }
 
+bool debugger_client::test_set_exception_breakpoints()
+{
+    attach();
+    nl::json reply = set_exception_breakpoints();
+    std::string code = "a = 3 * undef";
+    m_client.send_on_shell("execute_request", make_execute_request(code));
+
+    nl::json ev = m_client.wait_for_debug_event("stopped");
+    std::string reason = ev["content"]["body"]["reason"].get<std::string>();
+
+    m_client.send_on_control("debug_request", make_disconnect_request(6));
+    nl::json rep = m_client.receive_on_control();
+
+    return reason == "exception";
+}
+
 bool debugger_client::test_source()
 {
     attach();
@@ -598,7 +658,7 @@ bool debugger_client::test_stack_trace()
     ++seq;
     nl::json stackframes = m_client.receive_on_control();
 
-    bool res = stackframes["content"]["body"]["stackFrames"].size() == 1;
+    bool res = stackframes["content"]["body"]["stackFrames"].size() != 0;
     continue_exec(seq);
     m_client.wait_for_debug_event("stopped");
     continue_exec(seq);
@@ -677,6 +737,82 @@ bool debugger_client::test_inspect_variables()
     return res;
 }
 
+std::string rich_inspect_class_def = R"RICH(
+class Person:
+    def __init__(self, name="John Doe", address="Paris", picture=""):
+        self.name = name
+        self.address = address
+        self.picture = picture
+
+    def _repr_mimebundle_(self, include=None, exclude=None):
+        return {
+            "text/html": """<img src="{}">
+                  <div><i class='fa-user fa'></i>: {}</div>
+                  <div><i class='fa-map fa'></i>: {}</div>""".format(self.picture, self.name, self.address) 
+        }
+)RICH";
+
+std::string rich_html = R"RICH(<img src="">
+                  <div><i class='fa-user fa'></i>: James Smith</div>
+                  <div><i class='fa-map fa'></i>: Boston</div>)RICH";
+std::string rich_html2 = R"RICH(<img src="">
+                  <div><i class='fa-user fa'></i>: John Smith</div>
+                  <div><i class='fa-map fa'></i>: NYC</div>)RICH";
+
+bool debugger_client::test_rich_inspect_variables()
+{
+    bool res = false;
+    {
+        m_client.send_on_shell("execute_request", make_execute_request(rich_inspect_class_def));
+        m_client.receive_on_shell();
+
+        std::string code = "james = Person(\"James Smith\", \"Boston\")";
+        m_client.send_on_shell("execute_request", make_execute_request(code));
+        m_client.receive_on_shell();
+
+        m_client.send_on_control("debug_request", make_rich_inspect_variables_request(0, "james"));
+        nl::json rep = m_client.receive_on_control();
+        nl::json data = rep["content"]["body"]["data"];
+        std::string html = data["text/html"].get<std::string>();
+        std::string plain = data["text/plain"].get<std::string>();
+        res = html == rich_html && !plain.empty();
+    }
+    bool res2 = true;
+    {
+        attach();
+        std::string code = "john = Person(\"John Smith\", \"NYC\")\ni = 4\nj = 2";
+        m_client.send_on_shell("execute_request", make_execute_request(code));
+        m_client.receive_on_shell();
+
+        m_client.send_on_control("debug_request", make_dump_cell_request(12, code));
+        nl::json dump_res = m_client.receive_on_control();
+        std::string path = dump_res["content"]["body"]["sourcePath"].get<std::string>();
+        m_client.send_on_control("debug_request", make_breakpoint_request(12, path, 2));
+        m_client.receive_on_control();
+
+        m_client.send_on_shell("execute_request", make_execute_request(code));
+
+        nl::json ev = m_client.wait_for_debug_event("stopped");
+
+        int seq = 14;
+        m_client.send_on_control("debug_request", make_stacktrace_request(seq, 1));
+        nl::json stackframes = m_client.receive_on_control();
+        int frame_id = stackframes["content"]["body"]["stackFrames"][0]["id"].get<int>();
+        m_client.send_on_control("debug_request", make_rich_inspect_variables_request(seq, "john", frame_id));
+        nl::json rep = m_client.receive_on_control();
+        m_client.send_on_control("debug_request", make_continue_request(seq, 1));
+        m_client.receive_on_control();
+        m_client.receive_on_shell();
+
+        nl::json data = rep["content"]["body"]["data"];
+        std::string html = data["text/html"].get<std::string>();
+        std::string plain = data["text/plain"].get<std::string>();
+        res = html == rich_html2 && !plain.empty();
+
+    }
+    return res && res2;
+}
+
 bool debugger_client::test_variables()
 {
     std::string code = "i=4\nj=i+4\nk=j-3\ni=k+2\n";
@@ -764,6 +900,12 @@ nl::json debugger_client::set_breakpoints()
     return m_client.receive_on_control();
 }
 
+nl::json debugger_client::set_exception_breakpoints()
+{
+    m_client.send_on_control("debug_request", make_exception_breakpoint_request(4));
+    return m_client.receive_on_control();
+}
+
 std::string debugger_client::get_external_path()
 {
     return get_current_working_directory() + "/external_code.py";
@@ -979,6 +1121,21 @@ TEST(debugger, set_breakpoints)
     }
 }
 
+TEST(debugger, set_exception_breakpoints)
+{
+    start_kernel();
+    start_timer();
+    zmq::context_t context;
+    {
+        debugger_client deb(context, KERNEL_JSON, "debugger_set_exception_breakpoints.log");
+        bool res = deb.test_set_exception_breakpoints();
+        deb.shutdown();
+        std::this_thread::sleep_for(2s);
+        EXPECT_TRUE(res);
+        notify_done();
+    }
+}
+
 TEST(debugger, source)
 {
     start_kernel();
@@ -1030,7 +1187,7 @@ TEST(debugger, stack_trace)
     start_timer();
     zmq::context_t context;
     {
-        debugger_client deb(context, KERNEL_JSON, "debugger_debug_info.log");
+        debugger_client deb(context, KERNEL_JSON, "debugger_stack_trace.log");
         bool res = deb.test_stack_trace();
         deb.shutdown();
         std::this_thread::sleep_for(2s);
@@ -1060,7 +1217,7 @@ TEST(debugger, inspect_variables)
     start_timer();
     zmq::context_t context;
     {
-        debugger_client deb(context, KERNEL_JSON, "debugger_debug_info.log");
+        debugger_client deb(context, KERNEL_JSON, "debugger_inspect_variables.log");
         bool res = deb.test_inspect_variables();
         deb.shutdown();
         std::this_thread::sleep_for(2s);
@@ -1069,13 +1226,28 @@ TEST(debugger, inspect_variables)
     }
 }
 
+TEST(debugger, rich_inspect_variables)
+{
+    start_kernel();
+    start_timer();
+    zmq::context_t context;
+    {
+        debugger_client deb(context, KERNEL_JSON, "debugger_rich_inspect_variables.log");
+        bool res = deb.test_rich_inspect_variables();
+        deb.shutdown();
+        std::this_thread::sleep_for(2s);
+        EXPECT_TRUE(res);
+        notify_done();
+    }
+}
+
 TEST(debugger, variables)
 {
     start_kernel();
     start_timer();
     zmq::context_t context;
     {
-        debugger_client deb(context, KERNEL_JSON, "debugger_debug_info.log");
+        debugger_client deb(context, KERNEL_JSON, "debugger_variables.log");
         bool res = deb.test_variables();
         deb.shutdown();
         std::this_thread::sleep_for(2s);
@@ -1083,4 +1255,3 @@ TEST(debugger, variables)
         notify_done();
     }
 }
-
